@@ -104,11 +104,12 @@ class HomogeneousFDN:
     def set_model(self, input_layer=None, output_layer=None):
         # set the input and output layers of the FDN model
         if input_layer is None:
-            input_layer = dsp.FFT(self.config_dict.nfft)
+            input_layer = dsp.FFT(self.config_dict.nfft, dtype=self.config_dict.dtype)
         if output_layer is None:
             output_layer = dsp.iFFTAntiAlias(
                 nfft=self.config_dict.nfft,
                 alias_decay_db=self.config_dict.alias_decay_db,
+                dtype=self.config_dict.dtype,
             )
 
         self.model = self.get_shell(input_layer, output_layer)
@@ -125,6 +126,7 @@ class HomogeneousFDN:
             requires_grad=self.config_dict.input_gain_grad,
             alias_decay_db=self.config_dict.alias_decay_db,
             device=self.config_dict.device,
+            dtype=self.config_dict.dtype,
         )
         output_gain = dsp.Gain(
             size=(1, self.N),
@@ -132,6 +134,7 @@ class HomogeneousFDN:
             requires_grad=self.config_dict.output_gain_grad,
             alias_decay_db=self.config_dict.alias_decay_db,
             device=self.config_dict.device,
+            dtype=self.config_dict.dtype,
         )
 
         # RECURSION
@@ -144,6 +147,7 @@ class HomogeneousFDN:
             requires_grad=self.config_dict.delays_grad,
             alias_decay_db=self.config_dict.alias_decay_db,
             device=self.config_dict.device,
+            dtype=self.config_dict.dtype,
         )
         # assign the required delay line lengths
         delays.assign_value(delays.sample2s(delay_lines))
@@ -156,6 +160,7 @@ class HomogeneousFDN:
             requires_grad=self.config_dict.mixing_matrix_grad,
             alias_decay_db=self.config_dict.alias_decay_db,
             device=self.config_dict.device,
+            dtype=self.config_dict.dtype,
         )
 
         # homogeneous attenuation
@@ -165,6 +170,7 @@ class HomogeneousFDN:
             requires_grad=self.config_dict.attenuation_grad,
             alias_decay_db=self.config_dict.alias_decay_db,
             device=self.config_dict.device,
+            dtype=self.config_dict.dtype,
         )
         attenuation.map = map_gamma(delay_lines)
         attenuation.assign_value(
@@ -328,7 +334,8 @@ class parallelFDNAccurateGEQ(dsp.parallelAccurateGEQ):
         alias_decay_db: float = 0.0,
         start_freq: float = 31.25,
         end_freq: float = 16000.0,
-        device=None
+        device=None, 
+        dtype=torch.float32
     ):
         assert (delays is not None), "Delays must be provided"
         self.delays = delays
@@ -342,7 +349,8 @@ class parallelFDNAccurateGEQ(dsp.parallelAccurateGEQ):
             alias_decay_db=alias_decay_db,
             start_freq=start_freq,
             end_freq=end_freq,
-            device=device
+            device=device, 
+            dtype=dtype
         )
 
 
@@ -394,7 +402,8 @@ class parallelGFDNAccurateGEQ(parallelFDNAccurateGEQ):
         alias_decay_db: float = 0.0,
         start_freq: float = 31.25,
         end_freq: float = 16000.0,
-        device=None
+        device=None,
+        dtype=torch.float32
     ):
         assert (delays is not None), "Delays must be provided"
         self.delays = delays
@@ -410,7 +419,7 @@ class parallelGFDNAccurateGEQ(parallelFDNAccurateGEQ):
             device=device
         )
         self.n_gains = self.size[0]
-        self.size = (self.n_groups * self.size[0], len(self.delays))
+        self.size = (self.n_groups * self.size[0],)
         self.param = torch.nn.Parameter(
             torch.empty(self.size, device=self.device), requires_grad=self.requires_grad
         )
@@ -420,13 +429,13 @@ class parallelGFDNAccurateGEQ(parallelFDNAccurateGEQ):
         r"""
         Computes the polynomial coefficients for the SOS section.
         """
-        a = torch.zeros((3, self.size[0]+1, len(self.delays)), device=self.device)
-        b = torch.zeros((3, self.size[0]+1, len(self.delays)), device=self.device)
+        a = torch.zeros((3, self.size[0]+self.n_groups, len(self.delays)), device=self.device)
+        b = torch.zeros((3, self.size[0]+self.n_groups, len(self.delays)), device=self.device)
         for n_i in range(len(self.delays)):
             for i_group in range(self.n_groups):
                 (
-                    b[:, i_group * (self.n_gains) : (i_group + 1) * self.n_gains, n_i],
-                    a[:, i_group * (self.n_gains) : (i_group + 1) * self.n_gains, n_i],
+                    b[:, i_group * (self.n_gains+1) : (i_group + 1) * (self.n_gains+1), n_i],
+                    a[:, i_group * (self.n_gains+1) : (i_group + 1) * (self.n_gains+1), n_i],
                 ) = accurate_geq(
                     target_gain=param[
                         i_group * self.n_gains : (i_group + 1) * self.n_gains, n_i
@@ -482,6 +491,7 @@ class parallelFDNGEQ(dsp.parallelGEQ):
         requires_grad: bool = False,
         alias_decay_db: float = 0.0,
         device: Optional[str] = None,
+        dtype=torch.float32
     ):
         assert (delays is not None), "Delays must be provided"
         self.delays = delays
@@ -497,7 +507,8 @@ class parallelFDNGEQ(dsp.parallelGEQ):
             map=map,
             requires_grad=requires_grad,
             alias_decay_db=alias_decay_db,
-            device=device
+            device=device, 
+            dtype=dtype
         )
 
     def get_poly_coeff(self, param):
@@ -516,14 +527,13 @@ class parallelFDNGEQ(dsp.parallelGEQ):
                     fs=self.fs,
                     device=self.device
                 )
-        b_aa = torch.einsum('p, pon -> pon', self.alias_envelope_dcy.to(torch.double), b.to(torch.double))
-        a_aa = torch.einsum('p, pon -> pon', self.alias_envelope_dcy.to(torch.double), a.to(torch.double))
+        b_aa = torch.einsum('p, pon -> pon', self.alias_envelope_dcy, b)
+        a_aa = torch.einsum('p, pon -> pon', self.alias_envelope_dcy, a)
         B = torch.fft.rfft(b_aa, self.nfft, dim=0)
         A = torch.fft.rfft(a_aa, self.nfft, dim=0)
         H_temp = torch.prod(B, dim=1) / (torch.prod(A, dim=1))
         H = torch.where(torch.abs(torch.prod(A, dim=1)) != 0, H_temp, torch.finfo(H_temp.dtype).eps*torch.ones_like(H_temp))
-        H_type = torch.complex128 if param.dtype == torch.float64 else torch.complex64
-        return H.to(H_type), B, A
+        return H, B, A
 
     def check_param_shape(self):
         assert (
@@ -560,6 +570,7 @@ class parallelFDNPEQ(Filter):
         requires_grad: bool = False,
         alias_decay_db: float = 0.0,
         device: Optional[str] = None,
+        dtype=torch.float32
     ):
         self.delays = delays
         self.is_twostage = is_twostage
@@ -576,12 +587,13 @@ class parallelFDNPEQ(Filter):
         self.center_freq_bias = f_min * (f_max / f_min) ** ((k - 1) / (self.n_bands - 1))
         self.alias_envelope_dcy = gamma ** torch.arange(0, 3, 1, device=device)
         super().__init__(
-            size=(self.n_bands+1 if self.is_twostage else self.n_band, 3, 1 if self.is_proportional else len(delays)),
+            size=(self.n_bands+1 if self.is_twostage else self.n_bands, 3, 1 if self.is_proportional else len(delays)),
             nfft=nfft,
             map=map,
             requires_grad=requires_grad,
             alias_decay_db=alias_decay_db,
             device=device,
+            dtype=dtype
         )
 
     def get_poly_coeff(self, param):
@@ -644,16 +656,13 @@ class parallelFDNPEQ(Filter):
                 type='highshelf'
             )
 
-        b_aa = torch.einsum("p, opn -> opn", self.alias_envelope_dcy.to(torch.double), b.to(torch.double))
-        a_aa = torch.einsum("p, opn -> opn", self.alias_envelope_dcy.to(torch.double), a.to(torch.double))
+        b_aa = torch.einsum("p, opn -> opn", self.alias_envelope_dcy, b)
+        a_aa = torch.einsum("p, opn -> opn", self.alias_envelope_dcy, a)
         B = torch.fft.rfft(b_aa, self.nfft, dim=1)
         A = torch.fft.rfft(a_aa, self.nfft, dim=1)
         H_temp = (torch.prod(B, dim=0) / (torch.prod(A, dim=0)))
-        # H_temp = (torch.prod(B, dim=0) / (torch.prod(A, dim=0)))
-
         H = torch.where(torch.abs(torch.prod(A, dim=0)) != 0, H_temp, torch.finfo(H_temp.dtype).eps*torch.ones_like(H_temp))
-        H_type = torch.complex128 if param.dtype == torch.float64 else torch.complex64
-        return H.to(H_type), B, A
+        return H, B, A
 
     def compute_biquad_coeff(self, f, R, G, type='peaking'):
         # f : freq, R : resonance, G : gain in dB
@@ -805,6 +814,7 @@ class parallelFirstOrderShelving(dsp.parallelFilter):
         delays: torch.Tensor =  None,
         alias_decay_db: float = 0.0,
         device: str = None,
+        dtype: torch.dtype = torch.float32
     ):
         size = (2,)      # rt at DC and crossover frequency
         assert (delays is not None), "Delays must be provided"
@@ -816,7 +826,8 @@ class parallelFirstOrderShelving(dsp.parallelFilter):
             nfft=nfft,
             map=map,
             alias_decay_db=alias_decay_db,
-            device=device
+            device=device,
+            dtype=dtype
         )
         gamma = 10 ** (
             -torch.abs(torch.tensor(alias_decay_db, device=device)) / (nfft) / 20
