@@ -58,7 +58,7 @@ class LossProfile:
 
     """
 
-    def __init__(self, net: Shell, loss_config: LossConfig, device: str = "cpu"):
+    def __init__(self, net: Shell, loss_config: LossConfig, device: str = "cpu", dtype: torch.dtype = torch.float32):
 
         super().__init__()
         self.net = net
@@ -68,6 +68,7 @@ class LossProfile:
         self.n_runs = loss_config.n_runs
         self.output_dir = loss_config.output_dir
         self.device = device
+        self.dtype = dtype
         self.register_steps()
 
     def compute_loss(self, input: torch.Tensor, target: torch.Tensor):
@@ -101,9 +102,9 @@ class LossProfile:
                     if type(self.param_config.lower_bound) == list:
                         # interpolate between the lower and upper bound
                         new_value = (1 - steps[i_step]) * torch.tensor(
-                            self.param_config.lower_bound, device=self.device
+                            self.param_config.lower_bound, device=self.device, dtype=self.dtype
                         ) + steps[i_step] * torch.tensor(
-                            self.param_config.upper_bound, device=self.device
+                            self.param_config.upper_bound, device=self.device, dtype=self.dtype
                         )
                     else:
                         new_value = steps[i_step]
@@ -141,6 +142,7 @@ class LossProfile:
                 figsize=((len(self.criteria) + 2) // 3 * 5, 15),
             )
         steps = self.steps
+        steps = steps.cpu().numpy()
         for i_crit in range(len(self.criteria)):
             mean_loss = loss[:, :, i_crit].mean(0)
             std_loss = loss[:, :, i_crit].std(0)
@@ -230,12 +232,14 @@ class LossProfile:
             upper_bound = param_upper_bound
 
         if scale == "linear":
-            steps = torch.linspace(lower_bound, upper_bound, n_steps)
+            steps = torch.linspace(lower_bound, upper_bound, n_steps, device=self.device, dtype=self.dtype)
         elif scale == "log":
             steps = torch.logspace(
-                torch.log10(torch.tensor(lower_bound, device=self.device)),
-                torch.log10(torch.tensor(upper_bound, device=self.device)),
+                torch.log10(torch.tensor(lower_bound, device=self.device, dtype=self.dtype)),
+                torch.log10(torch.tensor(upper_bound, device=self.device, dtype=self.dtype)),
                 n_steps,
+                device=self.device,
+                dtype=self.dtype,
             )
         else:
             raise ValueError("Scale must be either 'linear' or 'log'")
@@ -305,6 +309,7 @@ class LossProfile:
         audio distances are bad at pitch,” arXiv preprint arXiv:2012.04572, 2020.
         """
         steps = self.steps
+        steps = steps.cpu().numpy()
         # find the index in steps of the element closest to the target value
         target_indx = np.abs(steps - self.param_config.target_value).argmin()
         accuracy = np.empty(loss.shape)
@@ -336,9 +341,9 @@ class LossSurface(LossProfile):
         - **steps** (dict): Dictionary of steps between the lower and upper bound of the parameters.
     """
 
-    def __init__(self, net: Shell, loss_config: LossConfig, device: str = "cpu"):
+    def __init__(self, net: Shell, loss_config: LossConfig, device: str = "cpu", dtype: torch.dtype = torch.float32):
 
-        super().__init__(net, loss_config, device)
+        super().__init__(net, loss_config, device, dtype)
 
         assert (
             len(loss_config.param_config) == 2
@@ -403,9 +408,9 @@ class LossSurface(LossProfile):
                     if type(self.param_config[0].lower_bound) == list:
                         # interpolate between the lower and upper bound
                         new_value = (1 - steps_0[i_step_0]) * torch.tensor(
-                            self.param_config[0].lower_bound, device=self.device
+                            self.param_config[0].lower_bound, device=self.device, dtype=self.dtype
                         ) + steps_0[i_step_0] * torch.tensor(
-                            self.param_config[0].upper_bound, device=self.device
+                            self.param_config[0].upper_bound, device=self.device, dtype=self.dtype
                         )
                     else:
                         new_value = steps_0[i_step_0]
@@ -419,9 +424,9 @@ class LossSurface(LossProfile):
                         if type(self.param_config[1].lower_bound) == list:
                             # interpolate between the lower and upper bound
                             new_value = (1 - steps_1[i_step_1]) * torch.tensor(
-                                self.param_config[1].lower_bound, device=self.device
+                                self.param_config[1].lower_bound, device=self.device, dtype=self.dtype
                             ) + steps_1[i_step_1] * torch.tensor(
-                                self.param_config[1].upper_bound, device=self.device
+                                self.param_config[1].upper_bound, device=self.device, dtype=self.dtype
                             )
                         else:
                             new_value = steps_1[i_step_1]
@@ -467,7 +472,7 @@ class LossSurface(LossProfile):
         steps_0 = self.steps_0
         steps_1 = self.steps_1
         # create a meshgrid
-        steps_0, steps_1 = np.meshgrid(steps_0, steps_1)
+        steps_0, steps_1 = np.meshgrid(steps_0.cpu().numpy(), steps_1.cpu().numpy())
         title = ["mean", "std"]
         for i_plot in range(2):
             if len(self.criteria) == 1:
@@ -484,9 +489,9 @@ class LossSurface(LossProfile):
                 )
             for i_crit in range(len(self.criteria)):
                 if i_plot == 0:
-                    x = loss[..., i_crit].mean(0)
+                    x = loss[..., i_crit].mean(0).T
                 else:
-                    x = loss[..., i_crit].std(0)
+                    x = loss[..., i_crit].std(0).T
                 criterion_name = self.criteria[i_crit].name
                 if len(self.criteria) == 1:
 
@@ -621,7 +626,6 @@ class LossSurface(LossProfile):
             plt.savefig(
                 f"{self.output_dir}/{title[i_plot]}_{self.param_config[0].key}_{self.param_config[1].key}.png"
             )
-            plt.clf()
         return fig, ax
 
     def compute_accuracy(self, loss):
@@ -632,6 +636,8 @@ class LossSurface(LossProfile):
         """
         steps_0 = self.steps_0
         steps_1 = self.steps_1
+        steps_0 = steps_0.cpu().numpy()
+        steps_1 = steps_1.cpu().numpy()
         # find the index in steps of the element closest to the target value
         target_indx_0 = np.abs(steps_0 - self.param_config[0].target_value).argmin()
         target_indx_1 = np.abs(steps_1 - self.param_config[1].target_value).argmin()
